@@ -11,13 +11,10 @@ use RPC::XML::Client;
 use RPC::XML;
 use MIME::Types;
 
-    use Data::Dumper;
+use Fcntl qw(:flock);
 
-    $Data::Dumper::Indent = 1;
-    $Data::Dumper::Terse = 1;
-    $Data::Dumper::Useqq = 1;
-    $Data::Dumper::Deepcopy = 1;
-    $Data::Dumper::Maxdepth = 0;
+# international symbols in our commands
+$RPC::XML::ENCODING = "UTF-8";
 
 my $SIZE_BY_CHUNKS_LIMIT=1024**3;
 
@@ -27,11 +24,11 @@ RTPG - is a module for accessing to rtorrent's SCGI functions.
 
 =head1 VERSION
 
-0.3
+0.92
 
 =cut
 
-our $VERSION=0.9;
+our $VERSION=0.92;
 
 =head1 SYNOPSIS
 
@@ -78,6 +75,10 @@ The constructor. It receives the next options:
 
 is an address of rtorrent's SCGI (direct) or rtorrent's RPC (standard).
 
+=item B<queue>
+
+if TRUE, commands will process in queue mode (use flock).
+
 =back
 
 =cut
@@ -100,7 +101,8 @@ sub new
         return bless {
             standard            =>  1,
             rtorrent_ctl_url    =>  $opts{url},
-            connection          =>  $connect
+            connection          =>  $connect,
+            queue_mode          =>  $opts{queue} || 0,
         }, $class;
     }
 
@@ -109,6 +111,7 @@ sub new
         standard            => 0,
         rtorrent_ctl_url    => $opts{url},
         connection          => $connect,
+        queue_mode          => $opts{queue} || 0,
     };
 }
 
@@ -132,7 +135,9 @@ sub rpc_command
     my ($cmd, @args)=@_;
     my $resp;
 
+    flock DATA, LOCK_EX if $self->{queue_mode};
     $resp=$self->{connection}->send_request($cmd, @args);
+    flock DATA, LOCK_UN if $self->{queue_mode};
 
     if (ref $resp)
     {
@@ -166,7 +171,7 @@ This method returns list of torrents. It is a link to array of hashes.
 
 =head3 EXAMPLE
 
-# standard version
+ # standard version
  my ($tlist, $err)=$h->torrents_list;
  my ($tlist, $err)=$h->torrents_list('started');
 
@@ -194,7 +199,7 @@ This method returns list of torrents. It is a link to array of hashes.
 
 =cut
 
-our $exclude_d_mask = qr{^d\.(get_mode|get_custom.*|get_bitfield)$};
+our $exclude_d_mask = qr{^d\.(?:get_mode|get_custom\D*|get_bitfield)$};
 
 sub torrents_list
 {
@@ -204,7 +209,7 @@ sub torrents_list
 
     my @iary=eval {
         grep !/$exclude_d_mask/,
-        grep /^d\.(get_|is_|views\.has$)/, $self->_get_list_methods;
+        grep /^d\.(?:get_|is_|views\.has$)/, $self->_get_list_methods;
     };
 
     if ($@)
@@ -261,6 +266,19 @@ sub torrents_list
 
         $_ = _normalize_one_torrent_info(\%info);
     }
+
+    # Set custom params count
+    if( @$list )
+    {
+        my $count = 0;
+        for my $name ( keys %{$list->[0]} )
+        {
+            next unless $name =~ '^custom(\d+)$';
+            $count = $1 if $1 > $count;
+        }
+        $_->{custom_count} = $count for @$list;
+    }
+
     return $list unless wantarray;
     return $list, '';
 }
@@ -525,7 +543,26 @@ sub set_file_priority
     }
 
     return $res;
+}
 
+=head2 update_priorities
+
+Update priorities before set_file_priority and etc.
+
+=cut
+
+sub update_priorities
+{
+    my ($self, $id) = @_;
+
+    my ($res, $error)= $self->rpc_command('d.update_priorities', $id);
+    unless (defined $res)
+    {
+        return undef, "$error" if wantarray;
+        die $error;
+    }
+
+    return $res;
 }
 
 =head2 system_information
@@ -549,26 +586,30 @@ the version of librtorrent.
 
 sub system_information
 {
-    my $self=shift;
+    my $self = shift;
 
-    my $lv;
-    my ($rv, $err)=$self->rpc_command('system.client_version');
-    ($lv, $err)=$self->rpc_command('system.library_version') if defined $rv;
+    my @info_params = qw(client_version library_version);
 
-    unless (defined $lv)
-    {
-        return undef, $err if wantarray;
-        die $err;
+    my ($res, $err) = $self->rpc_command(
+        'system.multicall', [
+            map { { methodName => "system.$_", params => [] } } @info_params
+        ]
+    );
+
+    if ($err) {
+        die $err unless wantarray;
+        return (undef, $err);
     }
 
-    my $res=
-    {
-        client_version      => $rv,
-        library_version     => $lv,
-    };
 
-    return $res, '' if wantarray;
-    return $res;
+    my %res;
+
+    for (0 .. $#info_params) {
+        $res{ $info_params[$_] } = $res->[$_][0];
+    }
+
+    return \%res, '' if wantarray;
+    return \%res;
 }
 
 =head2 view_list([ARGS])
@@ -840,13 +881,14 @@ sub rates
 
 =head2 add
 
-Add new torrent for download from url list or filehandle
+Add new torrent for download from url list or filehandle. If $start is TRUE then
+torrent immediately start after load.
 
 =cut
 
 sub add
 {
-    my ($self, $param) = @_;
+    my ($self, $param, $start) = @_;
 
     $param = [ $param ] unless 'ARRAY' eq ref $param;
 
@@ -859,12 +901,16 @@ sub add
             local $/;
             binmode $_;
             my $torrent = RPC::XML::base64->new(<$_>);
-            ($res, $error) = $self->rpc_command(load_raw => $torrent);
+            my $sub = 'load_raw';
+            $sub = 'load_raw_start' if $start;
+            ($res, $error) = $self->rpc_command($sub => $torrent);
         }
         else
         {
             my $url = RPC::XML::base64->new($_);
-            ($res, $error) = $self->rpc_command(load_verbose => $url);
+            my $sub = 'load';
+            $sub = 'load_start' if $start;
+            ($res, $error) = $self->rpc_command($sub => $url);
         }
     }
 
@@ -1117,3 +1163,6 @@ You should have received a copy of the GNU  General  Public  License  along
 with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 =cut
+
+__DATA__
+this is lock for this module
