@@ -27,14 +27,15 @@ sub new
     my ($class, %opts) = @_;
 
     # Get current state
-    $opts{$_} = cfg->get($_) for qw(locale horizontal vertical);
-
-    my $rtpg = RTPG->new(url => cfg->get('rpc_uri'));
-
-    $opts{methods} =[ $rtpg->_get_list_methods ] if cfg->get('debug');
+    $opts{$_} = cfg->get($_) for qw(layout locale horizontal vertical);
 
+    # Get sysinfo to check for rtorrent started and accessible
+    my $rtpg = RTPG->new(url => cfg->get('rpc_uri'), queue => 1);
     ($opts{info}, $opts{error}) = $rtpg->system_information;
 
+    # Get all methods list on debug
+    $opts{methods} =[ $rtpg->_get_list_methods ] if cfg->get('debug');
+
     my $self = bless \%opts, $class;
 
     return $self;
